/*
 * 2D Game Engine
 * Game.cpp: Base Game engine class
 * Based on pikuma.com 2D game engine in C++ and Lua course
 * Copyright (c) 2021 986-Studio. All Right Reserved
 *
 * Created by Manoël Trapier on 09/02/2021.
 */
#include <SDL.h>
#include <SDL_image.h>
#include <glm/glm.hpp>

#include <Game.h>
#include <Logger.h>
#include <ECS.h>

Game::Game()
{
    this->isRunning = false;
    this->windowsHeight = 0;
    this->windowsWidth = 0;
}

Game::~Game()
{
}

void Game::Initialize()
{
    int ret;
    SDL_DisplayMode displayMode;

    ret = SDL_Init(SDL_INIT_EVERYTHING);
    if (ret)
    {
        Logger::Error("SDL Initialisation error.");
        return;
    }

    SDL_GetCurrentDisplayMode(0, &displayMode);
    this->windowsWidth = displayMode.w;
    this->windowsHeight = displayMode.h;

    this->window = SDL_CreateWindow(NULL, SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED,
                                    this->windowsWidth, this->windowsHeight, SDL_WINDOW_BORDERLESS);
    if (!window)
    {
        Logger::Error("SDL window creation error.");
        return;
    }

    this->renderer = SDL_CreateRenderer(window, -1, 0);
    if (!renderer)
    {
        Logger::Error("SDL renderer creation error.");
        return;
    }

    SDL_SetWindowFullscreen(this->window, SDL_WINDOW_FULLSCREEN);

    this->isRunning = true;
}

void Game::Destroy()
{
    if (this->renderer)
    {
        SDL_DestroyRenderer(this->renderer);
    }
    if (this->window)
    {
        SDL_DestroyWindow(this->window);
    }

    SDL_Quit();
}

void Game::Run()
{
    this->Setup();

    while(this->isRunning)
    {
        this->ProcessInput();
        this->Update();
        this->Render();
    }
}

void Game::Setup()
{

}

void Game::Update()
{
    int32_t timeToWait = MILLISECS_PER_FRAME - (SDL_GetTicks() - this->millisecsPerviousFrame);
    if (timeToWait > 0 && timeToWait <= MILLISECS_PER_FRAME)
    {
        SDL_Delay(timeToWait);
    }
    double deltaTime = (SDL_GetTicks() - this->millisecsPerviousFrame) / 1000.0;

    this->millisecsPerviousFrame = SDL_GetTicks();
}

void Game::ProcessInput()
{
    SDL_Event event;

    while(SDL_PollEvent(&event))
    {
        switch(event.type)
        {
            case SDL_QUIT:
                this->isRunning = false;
                break;

            case SDL_KEYDOWN:
                if (event.key.keysym.sym == SDLK_ESCAPE)
                {
                    this->isRunning = false;
                }
                break;

            default:
                break;
        }
    }

}

void Game::Render()
{
    SDL_SetRenderDrawColor(this->renderer, 21, 21, 21, 255);
    SDL_RenderClear(this->renderer);



    SDL_RenderPresent(this->renderer);
}
